Subject: bug#25086: [PATCH] Fix the timezone detection of parse-iso8601-time-string.

From: Hong Xu <[email-masked]>
Date: Thu, 01 Dec 2016 20:04:00 -0800

On 2016-12-01 Thu 19:55 GMT-0800, Hong Xu <[email-masked]> wrote:

> On 2016-12-01 Thu 19:46 GMT-0800, Hong Xu <[email-masked]> wrote:
>
>> 	* parse-time.el (parse-iso8601-time-string): Fix its timezone
>>         parsing.
>>         * parse-time.el: Add doc for parse-iso8601-time-string and
>>         make it autoload.
>>         * editfns.c (Fdecode-time): Minor doc improvement for decode-time.
>
> I should have noted that the current version basically cannot parse the
> timezone at all. To reproduce, try the following:
>
> (format-time-string "%H %M %S"
>                     (parse-iso8601-time-string "1998-09-12T12:21:54-0200") t)
>
>
> Also missed one line:
>
> * emacs-mime.texi (time-date): Add an example for parse-iso8601-time-string.

I've now updated the patch and put everything together, as attached.

[PATCH] Fix the timezone detection of parse-iso8601-time-string.

        * parse-time.el (parse-iso8601-time-string): Fix its timezone
        parsing.
        * parse-time.el: Add doc for parse-iso8601-time-string and
        make it autoload.
        * editfns.c (Fdecode-time): Minor doc improvement for decode-time.
        * emacs-mime.texi (time-date): Add an example for
        parse-iso8601-time-string.
---
 doc/misc/emacs-mime.texi    |  3 +++
 lisp/calendar/parse-time.el | 12 ++++++++----
 src/editfns.c               |  5 ++---
 3 files changed, 13 insertions(+), 7 deletions(-)

diff --git a/doc/misc/emacs-mime.texi b/doc/misc/emacs-mime.texi
index 4d68246bba4b..8d6536265416 100644
--- a/doc/misc/emacs-mime.texi
+++ b/doc/misc/emacs-mime.texi
@@ -1536,6 +1536,9 @@ time-date
 (date-to-time "Sat Sep 12 12:21:54 1998 +0200")
 @result{} (13818 19266)
 
+(parse-iso8601-time-string "1998-09-12T12:21:54+0200")
+@result{} (13818 19266)
+
 (float-time '(13818 19266))
 @result{} 905595714.0
 
diff --git a/src/editfns.c b/src/editfns.c
index 5cc4a67ab19b..2f1f412d9e4e 100644
--- a/src/editfns.c
+++ b/src/editfns.c
@@ -2101,9 +2101,8 @@ format_time_string (char const *format, ptrdiff_t formatlen,
 
 DEFUN ("decode-time", Fdecode_time, Sdecode_time, 0, 2, 0,
        doc: /* Decode a time value as (SEC MINUTE HOUR DAY MONTH YEAR DOW DST UTCOFF).
-The optional SPECIFIED-TIME should be a list of (HIGH LOW . IGNORED),
-as from `current-time' and `file-attributes', or nil to use the
-current time.
+The optional TIME should be a list of (HIGH LOW . IGNORED), as from
+`current-time' and `file-attributes', or nil to use the current time.
 It can also be a single integer number of seconds since the epoch.
 The obsolete form (HIGH . LOW) is also still accepted.
 The optional ZONE is omitted or nil for Emacs local time, t for
diff --git a/lisp/calendar/parse-time.el b/lisp/calendar/parse-time.el
index 6ba26a4a00d0..e9dac0149b10 100644
--- a/lisp/calendar/parse-time.el
+++ b/lisp/calendar/parse-time.el
@@ -195,7 +195,7 @@ parse-time-iso8601-regexp
 	 (time-minute 2digit)
 	 (time-second 2digit)
 	 (time-secfrac "\\(\\.[0-9]+\\)?")
-	 (time-numoffset (concat "[-+]\\(" time-hour "\\):" time-minute))
+	 (time-numoffset (concat "\\([-+]" time-hour "\\):?" time-minute "?"))
 	 (time-offset (concat "Z" time-numoffset))
 	 (partial-time (concat time-hour colon time-minute colon time-second
 			       time-secfrac))
@@ -204,13 +204,17 @@ parse-time-iso8601-regexp
 	 (date-time (concat full-date "T" full-time)))
     (list (concat "^" full-date)
 	  (concat "T" partial-time)
-	  (concat "Z" time-numoffset)))
+	  (concat "Z?" time-numoffset)))
   "List of regular expressions matching ISO 8601 dates.
 1st regular expression matches the date.
 2nd regular expression matches the time.
 3rd regular expression matches the (optional) timezone specification.")
 
+;;;###autoload
 (defun parse-iso8601-time-string (date-string)
+  "Parse an ISO 8601 time string, such as 2016-12-01T23:35:06-05:00.
+If DATE-STRING cannot be parsed, it falls back to
+`parse-time-string'."
   (let* ((date-re (nth 0 parse-time-iso8601-regexp))
 	 (time-re (nth 1 parse-time-iso8601-regexp))
 	 (tz-re (nth 2 parse-time-iso8601-regexp))
@@ -235,10 +239,10 @@ parse-iso8601-time-string
                                                     "0"))
 	      re-start (match-end 0))
 	(when (string-match tz-re date-string re-start)
-	  (setq tz (match-string 1 date-string)))
+	  (setq tz (* 3600 (string-to-number (match-string 1 date-string)))))
 	(setq time (list seconds minute hour day month year day-of-week dst tz))))
 
     ;; Fall back to having Gnus do fancy things for us.
     (when (not time)
       (setq time (parse-time-string date-string)))
 
@@ -235,10 +239,10 @@ parse-iso8601-time-string
 	(setq time (list seconds minute hour day month year day-of-week dst tz))))
 
-    ;; Fall back to having Gnus do fancy things for us.
+    ;; Fall back to having `parse-time-string' do fancy things for us.
     (when (not time)
       (setq time (parse-time-string date-string)))
-- 
2.1.4